#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'Marga Millet < millet@cisco.com>'

import sys
import os
import yaml
import time
import amulet
import unittest
import paramiko
from collections import OrderedDict

"""
    IMPORTANT: Make sure you execute this script in an environment with
    at least two physical nodes (non virtual machines) and using the tag
    called "physical".
"""

VSM_AMULET_FILE = os.environ.get("VSM_AMULET_FILE", None)
if not VSM_AMULET_FILE:
    raise Exception("Please define VSM_AMULET_FILE environment variable "
                    "with path to the amulet.yaml file config for amulet.")

"""
    This need a yaml file with the configuration for the amulet testing.
    Similar to config.yaml of the VSM charm.
    Example:
           export VSM_AMULET_FILE = "~/amulet.yaml"
    and the amulet.yaml contains:
       vsm-primary:
           n1kv-source: "ppa:cisco-ppa/vsm"
           n1kv-vsm-domain-id: 10
           n1kv-vsm-password: "mypassw0rd"
           n1kv-vsm-mgmt-ip: "10.10.10.10"
           n1kv-vsm-mgmt-netmask: "255.255.255.0"
           n1kv-vsm-mgmt-gateway: 10.10.10.1"
           n1kv-phy-intf-bridge: "eth0"
"""
try:
    f = open(VSM_AMULET_FILE, 'r')
except IOError:
    msg = 'Error: Cannot open %s.' % VSM_AMULET_FILE
    amulet.raise_status(amulet.FAIL, msg=msg)
else:
    config = yaml.load(f)
    n1kv_source = config["vsm-primary"]["n1kv-source"]
    vsm_admin_pwd = config["vsm-primary"]["n1kv-vsm-password"]
    vsm_domain_id = config["vsm-primary"]["n1kv-vsm-domain-id"]
    vsm_mgmt_ip = config["vsm-primary"]["n1kv-vsm-mgmt-ip"]
    vsm_mgmt_mask = config["vsm-primary"]["n1kv-vsm-mgmt-netmask"]
    vsm_gateway = config["vsm-primary"]["n1kv-vsm-mgmt-gateway"]
    vsm_phy_intf = config["vsm-primary"]["n1kv-phy-intf-bridge"]
    f.close()


class VSMTest(unittest.TestCase):

    def setUp(self):
        pass

    def test_deployment_primary_vsm(self):
        """Test the deployment of the VSM VM"""
        self.deployment = amulet.Deployment(series="trusty", sentries=False)
        self.deployment.add("vsm-primary", charm="vsm",
                            constraints=OrderedDict([("tags", "physical")]))
        self.deployment.configure("vsm-primary", {
            'n1kv-source': n1kv_source,
            'n1kv-vsm-domain-id': vsm_domain_id,
            'n1kv-vsm-password': vsm_admin_pwd,
            'n1kv-vsm-mgmt-ip': vsm_mgmt_ip,
            'n1kv-vsm-mgmt-netmask': vsm_mgmt_mask,
            'n1kv-vsm-mgmt-gateway': vsm_gateway,
            'n1kv-phy-intf-bridge': vsm_phy_intf,
        })
        seconds = 800

        try:
            self.deployment.setup(timeout=seconds)
        except amulet.helpers.TimeoutError:
            msg = 'The environment did not setup in %d seconds.' % seconds
            amulet.raise_status(amulet.SKIP, msg=msg)
        except:
            raise

        # Wait for VSM to come up
        time.sleep(180)
        paramiko.common.logging.basicConfig(level=paramiko.common.DEBUG)
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        try:
            ssh.connect(vsm_mgmt_ip, username="admin", password=vsm_admin_pwd,
                        timeout=120)
        except Exception as e:
            err = 'Error(%s): Unable to ssh to VSM VM (%s)' % (e, vsm_mgmt_ip)
            amulet.raise_status(amulet.SKIP, msg=err)

        stdin, stdout, stderr = ssh.exec_command("show module")
        msg = stdout.readlines()
        if str(msg).find('active *') == -1:
            err = 'Error: VSM is not Active (%s)' % (msg)
            amulet.raise_status(amulet.SKIP, msg=err)
        ssh.close()

    def test_deployment_standby_vsm(self):
        """Test the deployment of a secondary VSM VM.
           This test cases assumes that test_deployment_primary_vsm has been
           executed first. Hence the primary VSM VM is running.
        """
        self.deployment = amulet.Deployment(series="trusty", sentries=False)
        self.deployment.add("vsm-secondary", charm="vsm",
                            constraints=OrderedDict([("tags", "physical")]))
        self.deployment.configure("vsm-secondary", {
            'n1kv-source': n1kv_source,
            'n1kv-vsm-domain-id': vsm_domain_id,
            'n1kv-vsm-name': 'vsm-s',
            'n1kv-vsm-role': 'secondary',
            'n1kv-vsm-password': vsm_admin_pwd,
            'n1kv-phy-intf-bridge': vsm_phy_intf,
        })
        seconds = 800

        try:
            self.deployment.setup(timeout=seconds)
        except amulet.helpers.TimeoutError:
            msg = 'The environment did not setup in %d seconds.' % seconds
            amulet.raise_status(amulet.SKIP, msg=msg)
        except:
            raise

        # Wait for VSM to come up and connect with the primary
        time.sleep(300)

        # Connect to the primary VSM and verify that it has conectivity
        # with the secondary
        paramiko.common.logging.basicConfig(level=paramiko.common.DEBUG)
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        try:
            ssh.connect(vsm_mgmt_ip, username="admin", password=vsm_admin_pwd)
        except Exception as e:
            err = 'Error(%s): No ssh to primary VSM VM (%s)' % (e, vsm_mgmt_ip)
            amulet.raise_status(amulet.SKIP, msg=err)

        stdin, stdout, stderr = ssh.exec_command("show module")
        msg = stdout.readlines()
        if str(msg).find('ha-standby') == -1:
            err = 'Error: VSM is not Standby VM (%s)' % (msg)
            amulet.raise_status(amulet.SKIP, msg=err)
        ssh.close()

if __name__ == "__main__":
    unittest.main()
